# Copyright (C) 2006 Canonical Limited.
# Authors: David Allouche <david@allouche.net>
#          Aaron Bentley <aaron.bentley@utoronto.ca>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
from bzrlib.errors import BzrCommandError
from bzrlib.commands import Command, register_command
from bzrlib.branch import Branch, BranchFormat
from bzrlib.bzrdir import BzrDir, BzrDirFormat
from bzrlib.transport import get_transport
from bzrlib.builtins import cmd_update
from bzrlib.trace import note
from bzrlib.workingtree import WorkingTree

class cmd_switch(Command):
    """Set the branch of a lightweight checkout and update.  <BZRTOOLS>"""

    takes_args = ['to_location']

    def run(self, to_location):
        to_branch = Branch.open(to_location)
        url = '.'
        wt = WorkingTree.open_containing(url)[0]
        branch = wt.branch
        unlockables = []
        try:
            wt.lock_write()
            unlockables.append(wt)
            branch.lock_write()
            unlockables.append(branch)
            self._check_switch_branch_format(url)
            self.set_branch_location(wt.bzrdir, to_branch.base)
            note('Switched to branch: %s' % (to_branch.base,))
            wt.update()
            note('Updated to revision %d' %
                 (wt.branch.revision_id_to_revno(wt.last_revision()),))
        finally:
            for unlockable in unlockables:
                unlockable.unlock()

    def _check_switch_branch_format(self, url):
        transport = get_transport(url)
        format = BzrDirFormat.find_format(transport)
        format_string = format.get_format_string()
        if not format_string.startswith("Bazaar-NG meta directory, "):
            raise BzrCommandError(
                'The switch command can only be used on a light checkout.\n'
                'Expected metadir, found %s at %s' % (
                format_string.strip(), transport.base))
        control = BzrDir.open_containing_from_transport(transport)[0]
        branch_format = BranchFormat.find_format(control)
        format_string = branch_format.get_format_string()
        if not format_string.startswith("Bazaar-NG Branch Reference Format "):
            raise BzrCommandError(
                'The switch command can only be used on a light checkout.\n'
                'Expected branch reference, found %s at %s' % (
                format_string.strip(), transport.base))
        if not format_string == "Bazaar-NG Branch Reference Format 1\n":
            raise BzrCommandError(
                'Unsupported: %r' % (format_string.strip(),))        

    def set_branch_location(self, control, location):
        """Set location value of a branch reference.

        The branch living in the control (BzrDir) object must be locked for
        writing.

        :param control: BzrDir containing the branch reference
        :param location: value to write to the branch reference location.
        """
        branch_format = BranchFormat.find_format(control)
        transport = control.get_branch_transport(None)
        branch = branch_format.open(control)
        location = transport.put('location', location)
